// Counter.java

/*
This software is part of the Aplu FtController package.
It is Open Source Free Software, so you may
- run the code for any purpose
- study how the code works and adapt it to your needs
- integrate all or parts of the code in your own programs
- redistribute copies of the code
- improve the code and release your improvements to the public
However the use of the code is entirely your responsibility.
 */

package ch.aplu.ftcontroller;

/**
 * Class to handle the eight impulse counters (at ports I1..I8).
 * Each time a connected switch changes its state (closed/opened), the counter
 * value is increased by 1.<br><br>
 * To clear all counters use FtController.clearAllCounters().
 * @see ch.aplu.ftcontroller.FtController
 */
public class Counter
{
  private FtController fc;
  private int counterNb;

  protected Counter(FtController fc, int counterNb)
  {
    this.fc = fc;
    this.counterNb = counterNb;
  }
  
  /**
   * Polls the counter and returns its current count.
   * @return the current count
   */
  public int getCount()
  {
    fc.counterNb = counterNb;
    int rc;
    synchronized(fc)
    {
      rc = fc.nh.invoke(7);
    }
    return rc;
  }

  /**
   * Sets the current count to 0.
   * @return a reference to the counter to allow chaining
   */
  public Counter clearCount()
  {
    return setCount(0);
  }

  /**
   * Sets the current count.
   * @param count the new current count
   * @return a reference to the counter to allow chaining
   */
  public Counter setCount(int count)
  {
    fc.counterNb = counterNb;
    fc.count = count;
    synchronized(fc)
    {
      fc.nh.invoke(8);
    }
    return this;
  }
}
